from matplotlib import pyplot as plt
from cartopy import crs as ccrs
import cartopy.feature as cfeature
import numpy as np
from matplotlib import colors as mcolors
from scipy import ndimage as ndi
import xarray as xr
from pyproj import Geod
from shapely.geometry import Polygon
from shapely.ops import orient

binary_grey = mcolors.ListedColormap(['white', 'grey'])

def plot_timestep(ds, file_name):

    fig, ax = plt.subplots(nrows=1, ncols=1, figsize=(11, 4), subplot_kw=dict(projection=ccrs.PlateCarree()))

    levels = np.linspace(97000, 104000, 14)

    ds.storm_areas.plot.contourf(levels=[0.5, 1000], cmap=binary_grey, add_colorbar=False) # , transform=ccrs.NorthPolarStereo())
    ds.msl.plot.contour(levels=levels, vmin=0, extend='max', cmap='Blues')

    ax.coastlines()
    ax.add_feature(cfeature.BORDERS.with_scale('50m'))
    # ax.set_extent(extent, crs=ccrs.PlateCarree())
    # yt1 = ax.set_yticks(y_ticks, crs=ccrs.PlateCarree())
    # xt1 = ax.set_xticks(x_ticks, crs=ccrs.PlateCarree())



    print("Save to " + file_name)
    plt.savefig(file_name, format='png')

    plt.figure().clear()
    plt.close()
    plt.cla()
    plt.clf()


def get_min_pressure_hPa(storm_area, msl_data):
    wh = msl_data.data[storm_area.data]
    return np.amin(wh) / 100.0

def get_msl_areas_by_hPa_thr(dataset, hPa):
    return dataset < hPa * 100.0

def split_areas(areas):
    # split dataarray areas into list of cohesive regions (DAs)
    lp_area_lab = xr.zeros_like(areas, dtype=int)

    lp_area_lab.data, num_features = ndi.label(areas)
    a_list = []
    for i in range(1, num_features + 1):
        a_list.append(lp_area_lab == i)

    return a_list


def get_storm_size_km2(storm_da):

    # create contour
    res = storm_da.plot.contour(levels=[0.5, 1000])
    paths = res.collections[0].get_paths()

    if len(paths) > 1:
        print("Multi path?") # TODO
    path = paths[0]

    #for v_idx in range(len(path.vertices) - 1):
    #            start_lonlat = path.vertices[v_idx][0], path.vertices[v_idx][1]
    #            end_lonlat = path.vertices[v_idx + 1][0], path.vertices[v_idx + 1][1]

    node_list = path.vertices

    polygon = Polygon(node_list)
    geod = Geod(ellps="WGS84")
    poly_area, poly_perimeter = geod.geometry_area_perimeter(orient(polygon))

    return poly_area / 1000000.0 # in km^2

def has_local_minimum(storm_area, msl_data):

    all_msl = msl_data.data[storm_area.data]
    min_all_data = np.amin(all_msl)

    inner_area = ndi.binary_erosion(storm_area.data)
    inner_msl = msl_data.data[inner_area]

    if inner_msl.size == 0:
        return False # too small

    min_inner_data = np.amin(inner_msl)
    return min_all_data == min_inner_data # true if min not on border


def get_storm_amplitude_hPa(storm_area, msl_data):

    all_msl = msl_data.data[storm_area.data]
    min_p = np.amin(all_msl)
    avg_p = np.mean(all_msl)
    ampl_p = avg_p - min_p

    return ampl_p / 100.0

